local StacKings = _G.StacKings

local debug = StacKings.debug

local barDefaults = {
	enabled = true,
	name = "New Bar",
	width = 100,
	height = 15,
	unit = "player",
	spell = "",
	auratype = "HELPFUL",
	spec = 0,
	bartexture = "Blizzard",
	fontface = "Friz Quadrata TT",
	fontsize = 12,
	fontcolor = {1, 1, 1},
	fontoutline = nil,
	color = {1,1,1},
	bgcolor = {0.3, 0.3, 0.3},
	bgalpha = 0.8,
	iconenable = true,
	max = 10,
	showTime = true,
	point = { "CENTER", nil, "CENTER", 0, 0 },
	showthreshold = 60,
	coloroverride = {},
}

local bars = {
	["SHAMAN"] = {
		[-1] = {
			name = GetSpellInfo(53817), -- Maelstrom Weapon
			spell = GetSpellInfo(53817), -- Maelstrom Weapon
			color = {0,0,1},
			icontexture = select(3, GetSpellInfo(53817)),
			max = 5,
		},
		[-2] = {
			name = GetSpellInfo(324), -- Lightning Shield
			spell = GetSpellInfo(324), -- Lightning Shield
			color = {0,0,0.7},
			bgcolor = {1, 0.3, 0.3},
			max = 3,
			icontexture = GetSpellInfo(324),
			point = { "CENTER", nil, "CENTER", 0, -30 },
		},
		[-3] = {
			name = GetSpellInfo(52127), -- Water Shield
			spell = GetSpellInfo(52127), -- Water Shield
			color = {0.2, 0.8, 1},
			bgcolor = {1, 0.3, 0.3},
			max = 4,
			icontexture = GetSpellInfo(52127),
			point = { "CENTER", nil, "CENTER", 0, -40 },
		},
	},
	["PRIEST"] = {
		[-1] = {
			name = GetSpellInfo(9573), -- Serendipity
			spell = GetSpellInfo(9573), -- Serendipity
			max = 2,
			icontexture = GetSpellInfo(9573), -- Serendipity
		},
	},
}

local function table_merge(tab, default)
	for k,v in pairs(default) do
		if not tab[k] then
			if type(v) == 'table' then
				tab[k] = CopyTable(v)
			else
				tab[k] = v
			end
		end
	end
	return tab
end

local _, class = UnitClass("player")
local classBars = bars[class]

if classBars then
	for k,v in pairs(classBars) do
		table_merge(v, barDefaults)
	end
else
	classBars = {}
end

local dbDefaults = {
	profile = {
		bars = classBars,
	},
}

local function getAura(unit, spell, auratype)
	if auratype == "HELPFUL" then
		return UnitAura(unit, spell)
	elseif auratype == "HARMFUL" then
		return UnitDebuff(unit, spell)
	end
end

local AceHook = LibStub("AceHook-3.0")
local iconfinderWorkspace = {}
local function aurahook()
	local update = false
	for i,v in ipairs(iconfinderWorkspace) do
		local _, _, icon = getAura(v.unit, v.spell, v.auratype)
		if icon then
			v.icontexture = icon
			table.remove(iconfinderWorkspace, i)
			update = true
		end
	end
	if not next(iconfinderWorkspace) then
		AceHook:Unhook(StacKings, "UNIT_AURA")
	end
	if update then
		StacKings:ApplyAllSettings()
	end
end
local function enableIconFinder(settings)
	if not AceHook:IsHooked(StacKings, "UNIT_AURA") then
		AceHook:Hook(StacKings, "UNIT_AURA", aurahook)
	end
	table.insert(iconfinderWorkspace, settings)
end

function StacKings:CheckForMissingIcons()
	for k,v in pairs(self.db.profile.bars) do
		if v.iconenable and (not v.icontexture or v.icontexture == "") then
			enableIconFinder(v)
		end
	end
end

function StacKings:CreateBar()
	local newIndex = #self.db.profile.bars + 1
	local newSettings = CopyTable(barDefaults)
	self.db.profile.bars[newIndex] = newSettings
	self:CreateBarFrame(newIndex)
	self:ApplyFrameSettings(newIndex, newSettings)
	self:UpdateBarOptions()
end

function StacKings:RemoveBar(name)
	self:RemoveBarFrame(name)
	wipe(self.db.profile.bars[name])
	self.db.profile.bars[name] = nil
	self:UpdateBarOptions()
end

function StacKings:ApplyAllSettings()
	self:ApplyAllFrameSettings(self.db.profile.bars)
end

function StacKings:OnProfileChanged()
	self:ApplyAllSettings()
	self:UpdateBarOptions()
	self:CheckForMissingIcons()
end

function StacKings:InitializeDB()
	local db = LibStub("AceDB-3.0"):New("StacKingsDB", dbDefaults)
	db.RegisterCallback(self, "OnProfileChanged", "OnProfileChanged")
	db.RegisterCallback(self, "OnProfileCopied", "OnProfileChanged")
	db.RegisterCallback(self, "OnProfileReset", "OnProfileChanged")
	self.db = db
	self:CheckForMissingIcons()
end
